@@ -335,7 +335,7 @@ class NormCipherFrame:
     def decrypt(self):
         decryptKeyDict = {t['id']:(t['atom_key'],t['atom_key_inv']) for t in decryption_key(
             self.apiContext,
-            json.dumps(flatten_list(flatten_list(self.indicesListOfListOfList)))
+            flatten_list(flatten_list(self.indicesListOfListOfList))
         )}
         decryptKey = [decryptKeyDict[i] for i in flatten_list(flatten_list(self.indicesListOfListOfList))]
         i=0
