@@ -133,7 +133,7 @@ class NormCipherQuant:
             
             decryptKeyDict = {t['id']:t['atom_key'] for t in decryption_key(
                 self.apiContext,
-                json.dumps(self.indicesList)
+                self.indicesList
             )}
             decryptKey = [decryptKeyDict[i] for i in self.indicesList]
             #print(decryptKey)
@@ -142,7 +142,7 @@ class NormCipherQuant:
             
             decryptKeyDict = {t['id']:(t['atom_key'],t['atom_key_inv']) for t in decryption_key(
                 self.apiContext,
-                json.dumps(self.indicesList)
+                self.indicesList
             )}
             decryptKey = [decryptKeyDict[i] for i in self.indicesList]
             #print(decryptKey)
@@ -308,7 +308,7 @@ class NormCipherQuant:
 
             plainResults = full_polynomial_compute(self.apiContext,'random-comparison',['x','y'],myIndices,cipherPair,isFloat=self.floatData,paillier=False,outPlain=True)
             
-            print(plainResults)
+            
             return [int(num < 0) for num in plainResults]
         elif isinstance(other, (int, float, complex,np.int64,np.float64)):
             other = NormCipherNum(self.apiContext,other,fromPlain=True,keyRange=20)
@@ -318,7 +318,7 @@ class NormCipherQuant:
             plainResults = full_polynomial_compute(self.apiContext,'random-comparison',
                                               ['x','y'],myIndices,cipherPair,isFloat=self.floatData,paillier=False,outPlain=True)
             
-            print(plainResults)
+            
             return [int(num < 0) for num in plainResults]
         else:
             return False
@@ -334,7 +334,7 @@ class NormCipherQuant:
             
             
             numList = [int(round(num) == 0) for num in plainResults]
-            return sum(numList) == len(numList)
+            return numList
         elif isinstance(other,NormCipherQuant):
             if len(other) != self.length:
                 return False
@@ -368,7 +368,7 @@ class NormCipherQuant:
             
             
             numList = [int(round(num) == 0) for num in plainResults]
-            return sum(numList) == len(numList)
+            return numList
         else:
             return False
     
