@@ -138,9 +138,22 @@ def encryption_key(apiContext,minMax,htmlDebug=False,seedString=False):
         print("Key generated locally from hash")
         return [{'id':t[0],'atom_key':t[1]} for t in zip(range(minMax[0],minMax[1]),string_to_key(seedString,minMax[1]-minMax[0]))]
     else:    
-        url = '/staticencrypt/?lower=%d&upper=%d' % (minMax[0],minMax[1],)
+        totalLength = minMax[1] - minMax[0]
+        requestLimit = 200000
+        fullResponse = []
+        for i in range((totalLength // requestLimit) + 1):
+            lower = minMax[0] + i * requestLimit
+            upper = minMax[0] + (i+1) * requestLimit
+            if lower == minMax[1]:
+                break
+            elif upper > minMax[1]:
+                upper = minMax[1]
+                
+            
+            url = '/staticencrypt/?lower=%d&upper=%d' % (lower,upper,)
+            fullResponse += apiContext.get(url,htmlDebug)
         #print(url)
-        return apiContext.get(url,htmlDebug)
+        return fullResponse
     
     
 def paillier_encryption_key(apiContext,htmlDebug=False,seedString=False):
@@ -154,20 +167,38 @@ def paillier_encryption_key(apiContext,htmlDebug=False,seedString=False):
     return apiContext.get(url,htmlDebug)[0]
 
 def decryption_key(apiContext,indicesJson,htmlDebug=False):
+    fullLength = len(indicesJson)
     
-    #get the current timestamp
-    timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
+    requestLimit = 200000
+    fullResponse = []
+    
+    #print(indicesJson)
+    for i in range((fullLength//requestLimit) + 1):
+        
+        
+        if i > fullLength // requestLimit:
+            curIndices = indicesJson[i*requestLimit:-1]
+        else:
+            curIndices = indicesJson[i*requestLimit:(i+1)*requestLimit]
+        #print(curIndices)
+        #get the current timestamp
+        if curIndices != []:
+            
+            timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
 
-    #post a blob of information about the desired polynomial computation at the given timestamp
-    
-    test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,"keyJSON":indicesJson,"userhash":timeStamp})
-    #print(test.text)
-    
-    url = '/decrypt/?blobData=%d' % (
-        timeStamp,
-    )
-    #print(url)
-    return apiContext.get(url,htmlDebug)
+            #post a blob of information about the desired polynomial computation at the given timestamp
+            test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,
+                                             "keyJSON":json.dumps(curIndices),"userhash":timeStamp})
+            #print(test.text)
+
+            url = '/decrypt/?blobData=%d' % (
+                timeStamp,
+            )
+            #print(url)  
+            response = apiContext.get(url)
+            fullResponse += response
+    #print(response)
+    return fullResponse
 
 def paillier_decryption_key(apiContext,index,htmlDebug=False):
     
@@ -192,7 +223,7 @@ def linking_key(apiContext,indicesJson):
 
     #post a blob of information about the desired polynomial computation at the given timestamp
     test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,"keyJSON":indicesJson,"userhash":timeStamp})
-    print(test)
+    #print(test)
     
     url = '/recordlink/?blobData=%d' % (
         timeStamp,
@@ -202,20 +233,38 @@ def linking_key(apiContext,indicesJson):
 
 def align_index_key(apiContext,indicesJson):
     
-    #get the current timestamp
-    timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
-
-    #post a blob of information about the desired polynomial computation at the given timestamp
-    test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,"keyJSON":json.dumps(indicesJson),"userhash":timeStamp})
-    #print(test.text)
+    fullLength = len(indicesJson)
+    requestLimit = 200000
+    fullResponse = []
     
-    url = '/align-indices/?blobData=%d' % (
-        timeStamp,
-    )
-    #print(url)    
-    return apiContext.get(url)
+    for i in range((fullLength//requestLimit) + 1):
+        
+        
+        if i > fullLength // requestLimit:
+            curIndices = indicesJson[i*requestLimit:-1]
+        else:
+            curIndices = indicesJson[i*requestLimit:(i+1)*requestLimit]
+        
+        #get the current timestamp
+        if curIndices != []:
+            
+            timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
 
-def ngram_checksum_key(apiContext,window,wordLength,indicesJson,isFloat=False):
+            #post a blob of information about the desired polynomial computation at the given timestamp
+            test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,
+                                             "keyJSON":json.dumps(curIndices),"userhash":timeStamp})
+            #print(test.text)
+
+            url = '/align-indices/?blobData=%d' % (
+                timeStamp,
+            )
+            #print(url)  
+            response = apiContext.get(url)
+            fullResponse += response
+    #print(response)
+    return fullResponse
+
+def ngram_checksum_key(apiContext,window,wordLength,indicesJson,isFloat=False,isAligned=False):
     
     #get the current timestamp
     timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
@@ -225,71 +274,167 @@ def ngram_checksum_key(apiContext,window,wordLength,indicesJson,isFloat=False):
     #print(test)
     
     #this url
-    url = '/ngramview/?win=%d&wordLength=%d&blobData=%d&isFloat=%s' % (
+    url = '/ngramview/?win=%d&wordLength=%d&blobData=%d&isFloat=%s&isAligned=%s' % (
         window,
         wordLength,
         timeStamp,
-        isFloat
+        isFloat,
+        isAligned
     )
+    #print(url)
     return apiContext.get(url)
 
 def polyn_comp_key(apiContext,polyn,polynVars,indicesTupleList,dualListOfList,isFloat=False,paillier=True):
+
+    fullLength = len(indicesTupleList) * len(indicesTupleList[0])
     
-    #get the current timestamp
-    timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
+    requestLimit = 20000
+    randomSeed = random.randint(1,1000000)
+    fullMonomial = None
+
+    singleCallLength = len(indicesTupleList[0])
+
+    for i in range(fullLength//requestLimit + 1):
+        
+        if (i+1)*requestLimit*singleCallLength > fullLength:
+            curIndices = indicesTupleList[i*(requestLimit//singleCallLength):]
+        elif i*requestLimit*singleCallLength >= fullLength:
+            pass
+        else:
+            curIndices = indicesTupleList[i*(requestLimit//singleCallLength):(i+1)*(requestLimit//singleCallLength)]
+
+        #get the current timestamp
+        timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
+        
+        #information about the polynomial to be computed
+        polynData = {
+            'polyn':polyn,
+            'polynVars':polynVars,
+            'indicesTupleList':curIndices,
+            'dualTupleList':dualListOfList,
+            'isFloat':isFloat,
+            'paillier':paillier,
+            'seed': randomSeed,
+        }
+        
+
+        #post a blob of information about the desired polynomial computation at the given timestamp
+        test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,"keyJSON":json.dumps(polynData),"userhash":timeStamp})
+        #print(test.text)
+        #this url tells the general polynomial endpoint to compute the desired polynomial
+        url = '/general/?blobData=%d' % (
+            timeStamp,
+        )
+        
+        output = apiContext.get(url)
+        if i == 0 :
+            fullMonomial = json.loads(output[0]['keyJSON'])['monomialData']
+        else:
+            
+            fullMonomial = {**fullMonomial,**json.loads(output[0]['keyJSON'])['monomialData']}
     
-    #information about the polynomial to be computed
-    polynData = {
-        'polyn':polyn,
-        'polynVars':polynVars,
-        'indicesTupleList':indicesTupleList,
-        'dualTupleList':dualListOfList,
-        'isFloat':isFloat,
-        'paillier':paillier,
-    }
+    finalOutput = output
+    fullJSON = json.loads(output[0]['keyJSON'])
+    fullJSON['monomialData'] = fullMonomial
+    fullJSON['indicesTupleList'] = indicesTupleList
+    finalOutput[0]['keyJSON'] = json.dumps(fullJSON)
+    #print(finalOutput)
+    return finalOutput
+
+def hash_key(apiContext,wordLength,indicesJson,n):
     
+    fullLength = len(indicesJson)
+    requestLimit = 500000
+    randomSeed = random.randint(1,1000000)
+    fullResponse = []
+   
     
-    #post a blob of information about the desired polynomial computation at the given timestamp
-    test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,"keyJSON":json.dumps(polynData),"userhash":timeStamp})
-    #print(test.text)
-    #this url tells the general polynomial endpoint to compute the desired polynomial
-    url = '/general/?blobData=%d' % (
-        timeStamp,
-    )
+    for i in range(int(fullLength/wordLength)//requestLimit + 1):
+        
+        
+        if (i+1)*requestLimit*wordLength > fullLength:
+            curIndices = indicesJson[i*requestLimit*wordLength:]
+        elif i*requestLimit*wordLength >= fullLength:
+            pass
+        else:
+            curIndices = indicesJson[i*requestLimit*wordLength:(i+1)*requestLimit*wordLength]
     
-    output = apiContext.get(url)
-    #print(output)
-    return output
+        #get the current timestamp
+        timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
+
+        #post a blob of information about the desired polynomial computation at the given timestamp
+        test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,"keyJSON":indicesJson,"userhash":timeStamp})
+        #print(test.text)
+
+        #this url
+
+        url = '/hash/?wordLength=%d&blobData=%d&n=%d&seed=%d' % (
+            wordLength,
+            timeStamp,
+            n,
+            randomSeed
+        )
+        fullResponse += apiContext.get(url)
+        
+    return fullResponse
+
 
 def rand_poly_comp_key(apiContext,polyn,polynVars,indicesTupleList,dualListOfList,isFloat=False,paillier=True):
+
+    fullLength = len(indicesTupleList) * len(indicesTupleList[0])
     
-    #get the current timestamp
-    timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
+    requestLimit = 20000
+    randomSeed = random.randint(1,1000000)
+    fullMonomial = None
     
-    #information about the polynomial to be computed
-    polynData = {
-        'polyn':polyn,
-        'polynVars':polynVars,
-        'indicesTupleList':indicesTupleList,
-        'dualTupleList':dualListOfList,
-        'isFloat':isFloat,
-        'paillier':paillier,
-    }
+    for i in range(fullLength//requestLimit + 1):
+        
+        if (i+1)*requestLimit*singleCallLength > fullLength:
+            curIndices = indicesTupleList[i*(requestLimit//singleCallLength):]
+        elif i*requestLimit*singleCallLength >= fullLength:
+            pass
+        else:
+            curIndices = indicesTupleList[i*(requestLimit//singleCallLength):(i+1)*(requestLimit//singleCallLength)]
+
+        #get the current timestamp
+        timeStamp = int(str(datetime.datetime.now()).replace(' ','').replace('-','').replace(':','').replace('.','')[0:18])
+        
+        #information about the polynomial to be computed
+        polynData = {
+            'polyn':polyn,
+            'polynVars':polynVars,
+            'indicesTupleList':indicesTupleList,
+            'dualTupleList':dualListOfList,
+            'isFloat':isFloat,
+            'paillier':paillier,
+            'seed': randomSeed,
+        }
+        
+        #print(polyn)
+        
+        #post a blob of information about the desired polynomial computation at the given timestamp
+        test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,"keyJSON":json.dumps(polynData),"userhash":timeStamp})
+        #print(test.text)
+        
+        #this url tells the general polynomial endpoint to compute the desired polynomial
+        url = '/randomized-poly/?blobData=%d' % (
+            timeStamp,
+        )
+        
+        output = apiContext.get(url)
+        if i == 0 :
+            fullMonomial = json.loads(output[0]['keyJSON'])['monomialData']
+        else:
+            
+            fullMonomial = {**fullMonomial,**json.loads(output[0]['keyJSON'])['monomialData']}
     
-    #print(polyn)
-    
-    #post a blob of information about the desired polynomial computation at the given timestamp
-    test = apiContext.post('/blob/',{"assigned_user":apiContext.userId,"keyJSON":json.dumps(polynData),"userhash":timeStamp})
-    #print(test.text)
-    
-    #this url tells the general polynomial endpoint to compute the desired polynomial
-    url = '/randomized-poly/?blobData=%d' % (
-        timeStamp,
-    )
-    
-    output = apiContext.get(url)
-    #print(output)
-    return output
+    finalOutput = output
+    fullJSON = json.loads(output[0]['keyJSON'])
+    fullJSON['monomialData'] = fullMonomial
+    fullJSON['indicesTupleList'] = indicesTupleList
+    finalOutput[0]['keyJSON'] = json.dumps(fullJSON)
+    #print(finalOutput)
+    return finalOutput
 
 def paillier_convert(apiContext,paillierDict):
     
